@@ -17,6 +17,12 @@ import pandas as pd
 from pyxllib.prog.pupil import dprint, typename
 from pyxllib.file.specialist import XlPath
 
+pd.options.display.unicode.east_asian_width = True  # 优化中文输出对齐问题
+try:
+    pd.set_option('future.no_silent_downcasting', True)
+except Exception as e:
+    pass
+
 
 def treetable(childreds, parents, arg3=None, nodename_colname=None):
     """ 输入childres子结点id列表，和parents父结点id列表
@@ -181,7 +187,7 @@ def write_dataframes_to_excel(outfile, dataframes, order_mode='序号'):
 
     实现上，尽可能在一些常见结构上，进行一些格式美化。但对费常规结构，就保留df默认排版效果，不做特殊处理。
     """
-    with pd.ExcelWriter(str(outfile)) as writer:
+    with pd.ExcelWriter(str(outfile), engine='xlsxwriter') as writer:
         head_format = writer.book.add_format({'font_size': 12, 'font_color': 'blue',
                                               'align': 'left', 'valign': 'vcenter'})
         for sheet_name, df in dataframes.items():
@@ -202,7 +208,8 @@ def write_dataframes_to_excel(outfile, dataframes, order_mode='序号'):
             # 设置表头格式
             if df.columns.nlevels == 1:
                 start = df.index.nlevels
-                if start == 1: start = 0
+                if start == 1:
+                    start = 0
                 for col_num, value in enumerate(df.columns, start=start):
                     writer.sheets[sheet_name].write(0, col_num, value, head_format)
 
@@ -247,13 +254,18 @@ def xlpivot(df, index=None, columns=None, values=None):
     :param columns: 列划分方式
     :param values: 显示的值
         Callable[items, value]：输出一个函数
+        list[str]: 支持输入属性列表，表示显示原始值的意思。如果原始值不唯一，则逗号分开拼接后显示。但这种用法就不太算是传统意义的数据透视表了
     :return: 数据透视表的表格
 
     使用示例：
-    def func(items):
+    def func(items):  # 输入匹配的多行数据
         x = items.iloc[0]
-        return f'{x["precision"]:.0f}，{x["recall"]:.0f}，{x["hmean"]:.2f}，{x["fps"]}'
-    >> df2 = xlpivot(df, ['model_type'], ['dataset', 'total_frame'], {'precision，recall，hmean，fps': func})
+        return f'{x["precision"]:.0f}，{x["recall"]:.0f}，{x["hmean"]:.2f}，{x["fps"]}'  # 返回显示的值
+
+    >> df2 = xlpivot(df, ['model_type'], ['dataset', 'total_frame'],
+                        {'precision，recall，hmean，fps': func})
+
+    注意技巧：如果要在分组后约束特定顺序，可以使用特殊前缀进行编号对齐
     """
 
     # 1 将分组的格式标准化
@@ -268,6 +280,17 @@ def xlpivot(df, index=None, columns=None, values=None):
     index_, columns_ = reset_groups(index), reset_groups(columns)
 
     # 2 目标值的格式标准化
+    def make_col_func(col):
+        def func(rows):
+            if len(rows):
+                return ', '.join(map(str, rows[col].values))
+            return ''
+
+        return func
+
+    if isinstance(values, (list, tuple)):
+        values = {v: make_col_func(v) for v in values}
+
     if callable(values):
         values_ = {'values': values}
     elif isinstance(values, dict):
@@ -276,6 +299,8 @@ def xlpivot(df, index=None, columns=None, values=None):
         raise TypeError
 
     # 3 分组
+    assert len(df), 'df是空的'
+
     keys = index_ + columns_
     dfgp = df.groupby(keys)
     data = defaultdict(list)
@@ -329,7 +354,7 @@ def count_key_combinations(df, col_names, count_col_name='count'):
     return df2
 
 
-def pareto_accumulate(weights, accuracy=0.01, *, print_mode=False):
+def pareto_accumulate(weights, accuracy=0.01, *, print_mode=False, value_unit_type='K'):
     """ 帕累托累计
 
     可以用来分析主要出现的权重、频次
@@ -337,7 +362,8 @@ def pareto_accumulate(weights, accuracy=0.01, *, print_mode=False):
 
     :param weights: 一组权重数据
     :param accuracy: 累计精度，当统计到末尾时，可能有大量权重过小的数值
-        此时不频繁进行累计权重计算，而是但更新权重累计达到accuracy，才会更新一个记录点
+        此时不频繁进行累计权重计算，而是当更新权重累计达到accuracy，才会更新一个记录点
+        注意这是全量数据综合的百分比，所以最小更新量就是1%
     :param print_mode: 是否直接展示可视化结果
     :return: [(累计数值数量, ≥当前阈值, 累计权重), ...]
 
@@ -366,9 +392,9 @@ def pareto_accumulate(weights, accuracy=0.01, *, print_mode=False):
 
     # 2 结果展示
     def fmt(p):
-        from pyxllib.text.newbie import format_big_decimal
-        ls = [f'{p[0]}it≥{format_big_decimal(p[1])}',
-              f'{format_big_decimal(p[2])}({p[2] / total_size:.0%})']
+        from pyxllib.prog.newbie import human_readable_number
+        ls = [f'{human_readable_number(p[0], "万")}条≥{human_readable_number(p[1])}',
+              f'{human_readable_number(p[2], value_unit_type)}({p[2] / total_size:.0%})']
         return '，'.join(map(str, ls))
 
     total_size = points[-1][2]
@@ -411,3 +437,58 @@ class ModifiableRow:
 
     def __setitem__(self, key, value):
         self.df.at[self.index, key] = value
+
+
+def print_full_dataframe(df):
+    """
+    临时设置以完整显示DataFrame的内容
+
+    :param pd.DataFrame df: 需要完整显示的DataFrame
+    """
+    with pd.option_context('display.max_rows', None,
+                           'display.max_columns', None,
+                           'display.width', 1000,
+                           'display.max_colwidth', None):
+        print(df)
+
+    pd.options.display.max_rows = 60
+
+
+def custom_fillna(df, default_fill_value='', numeric_fill_value=None, specific_fill=None):
+    """ 使用更多灵活性填充DataFrame中的NaN值。
+
+    :param pandas.DataFrame df: 需要处理的DataFrame。
+    :param str default_fill_value: 非数值列中NaN的默认填充值。
+    :param numeric_fill_value: 数值列中NaN的填充值，如果不指定，则默认为None。
+    :param dict specific_fill: 指定列名及其NaN的填充值，如果不指定，则默认为None。
+    :return: 已根据指定标准填充NaN值的pandas.DataFrame。
+
+    >>> df = pd.DataFrame({'A': [1, 2, None], 'B': [None, 'x', 'y'], 'C': [None, None, None]})
+    >>> custom_fillna(df, 'filled', 0, {'C': 'special'})
+    """
+    for column in df.columns:
+        # 检查列是否在specific_fill中指定；如果是，则使用指定的值填充。
+        if specific_fill and column in specific_fill:
+            df[column] = df[column].fillna(specific_fill[column])
+        # 如果列是数值型且指定了numeric_fill_value，则使用numeric_fill_value填充。
+        elif numeric_fill_value is not None and pd.api.types.is_numeric_dtype(df[column]):
+            df[column] = df[column].fillna(numeric_fill_value)
+        # 否则，对非数值列使用default_fill_value进行填充。
+        elif pd.api.types.is_object_dtype(df[column]) or pd.api.types.is_string_dtype(df[column]):
+            df[column] = df[column].fillna(default_fill_value)
+        # 可以在这里添加更多条件，以处理其他数据类型，如datetime。
+    return df
+
+
+def dataframe_to_list(df):
+    """将DataFrame转换为列表结构，第一行是表头，其余是数据"""
+    # 获取表头（列名）作为第一个列表元素
+    headers = df.columns.tolist()
+
+    # 获取数据行，每一行作为一个列表，然后将所有这些列表收集到一个大列表中
+    data_rows = df.values.tolist()
+
+    # 将表头和数据行合并成最终的列表
+    result_list = [headers] + data_rows
+
+    return result_list
