@@ -13,6 +13,7 @@ import humanfriendly
 import numpy as np
 import requests
 
+
 from pyxllib.prog.newbie import round_int, RunOnlyOnce
 from pyxllib.prog.pupil import EnchantBase, EnchantCvt
 from pyxllib.algo.geo import rect_bounds, warp_points, reshape_coords, quad_warp_wh, get_warp_mat, rect2polygon
@@ -112,6 +113,7 @@ class xlcv(EnchantBase):
     def read_from_strokes(strokes, margin=10, color=(0, 0, 0), bgcolor=(255, 255, 255), thickness=2):
         """ 将联机手写笔划数据转成图片
 
+        :param strokes: n个笔划，每个笔画包含不一定要一样长的m个点，每个点是(x, y)的结构
         :param margin: 图片边缘
         :param color: 前景笔划颜色，默认黑色
         :param bgcolor: 背景颜色，默认白色
@@ -135,7 +137,7 @@ class xlcv(EnchantBase):
         # 画出每个笔划轨迹
         for stroke in strokes:
             for i in range(len(stroke) - 1):
-                cv2.line(canvas, stroke[i], stroke[i + 1], color, thickness=thickness)
+                cv2.line(canvas, tuple(stroke[i]), tuple(stroke[i + 1]), color, thickness=thickness)
 
         return canvas
 
@@ -265,9 +267,11 @@ class xlcv(EnchantBase):
         return im
 
     @staticmethod
-    def write(im, file, if_exists=None):
+    def write(im, file, if_exists=None, ext=None):
         file = XlPath(file)
-        data = cv2.imencode(ext=file.suffix, img=im)[1]
+        if ext is None:
+            ext = file.suffix
+        data = cv2.imencode(ext=ext, img=im)[1]
         if file.exist_preprcs(if_exists):
             file.write_bytes(data.tobytes())
         return file
@@ -784,6 +788,21 @@ class xlcv(EnchantBase):
             images = [hstack(imgs) for imgs in images]
         return vstack(images)
 
+    @classmethod
+    def deskew_image(cls, image):
+        """ 歪斜图像矫正 """
+        # pip install deskew
+        from deskew import determine_skew  # 用来做图像倾斜矫正的，这个库不大，就自动安装了
+
+        gray = xlcv.reduce_area(xlcv.read(image, 0), 1000*1000)  # 转成灰度图，并且可以适当缩小计算图片
+        angle = determine_skew(gray)  # 计算图像的倾斜角度
+        (h, w) = image.shape[:2]  # 获取图像尺寸
+        center = (w // 2, h // 2)  # 计算图像中心（默认按中心旋转了，以后如果有需要按非中心点旋转的，可以再改）
+        M = cv2.getRotationMatrix2D(center, angle, 1.0)  # 计算旋转矩阵
+        # 旋转图像
+        deskewed_image = cv2.warpAffine(image, M, (w, h), flags=cv2.INTER_CUBIC, borderMode=cv2.BORDER_REPLICATE)
+        return deskewed_image  # 返回纠偏后的图像
+
     def __6_替换颜色(self):
         pass
 
