@@ -5,12 +5,21 @@
 # @Date   : 2023/08/02 14:05
 
 import os
+import random
 import re
 import sys
 import json
 import time
 from types import SimpleNamespace
 
+import warnings
+
+warnings.filterwarnings('ignore')
+
+import logging
+
+logging.disable(logging.CRITICAL)
+
 from PyQt5.QtWidgets import QApplication, QMainWindow, QAction, QFileDialog, QListWidget, QLineEdit, QVBoxLayout, \
     QSplitter, QTreeView, QPlainTextEdit, QPushButton, QLabel, QHBoxLayout, QSizePolicy, QWidget, QStatusBar, \
     QAbstractItemView, QHeaderView, QMessageBox
@@ -55,6 +64,9 @@ class MyTreeView(QTreeView):
     def edit(self, index, trigger, event):
         if trigger == QAbstractItemView.DoubleClicked:
             return False
+            # 如果是第 0 列 (keys)，则禁止编辑
+        if index.column() == 0:
+            return False
         return super().edit(index, trigger, event)
 
 
@@ -65,19 +77,18 @@ class KeyStandardItem(QStandardItem):
         return super().data(role)
 
 
-class TextEditDelegate(QItemDelegate):
+class ExpandedTextEditDelegate(QItemDelegate):
     def createEditor(self, parent, option, index):
         return QTextEdit(parent)
 
+    def sizeHint(self, option, index):
+        size = super().sizeHint(option, index)
+        return size
+
     def setEditorData(self, editor, index):
         value = index.model().data(index, Qt.EditRole)
         editor.setPlainText(value)
 
-    def sizeHint(self, option, index):
-        size = super().sizeHint(option, index)
-        size.setHeight(20)  # 限制最大高度为20像素，限定每个条目只展示一行
-        return size
-
     def setModelData(self, editor, model, index):
         value = editor.toPlainText()
         model.setData(index, value, Qt.EditRole)
@@ -85,13 +96,21 @@ class TextEditDelegate(QItemDelegate):
     def updateEditorGeometry(self, editor, option, index):
         editor.setGeometry(option.rect)
 
+
+class CompactTextEditDelegate(ExpandedTextEditDelegate):
+    def sizeHint(self, option, index):
+        size = super().sizeHint(option, index)
+        # todo 如果一个4k的屏幕使用2k的时候，这个缩放比例不会自动兼容，阁下又当如何应对
+        size.setHeight(20)  # 限制最大高度为20像素，限定每个条目只展示一行
+        return size
+
     def paint(self, painter, option, index):
         text = index.model().data(index)
 
         # 只显示前100个字符
-        elided_text = text[:100] + '...' if len(text) > 100 else text
+        text = text[:100] + '...' if len(text) > 100 else text
 
-        painter.drawText(option.rect, Qt.AlignLeft, elided_text)
+        painter.drawText(option.rect, Qt.AlignLeft, text)
 
 
 class JLineViewer(QMainWindow):
@@ -152,9 +171,16 @@ class JLineViewer(QMainWindow):
         openFile.setStatusTip('打开新文件，可以打开jsonl或json格式的文件')
         openFile.triggered.connect(self.showDialog)
 
-        self.loadAllButton = QPushButton("加载全部")
+        self.reloadButton = QAction('重新加载', self)
+        self.reloadButton.triggered.connect(self.reload)
+
+        self.loadAllButton = QAction("加载全部", self)
         self.loadAllButton.setStatusTip('对jsonl最多只会预加载1000行，点击该按钮可以加载剩余全部条目')
-        self.loadAllButton.clicked.connect(self.loadAllItems)
+        self.loadAllButton.triggered.connect(self.loadAllItems)
+
+        self.delegate_mode = 'compact'
+        self.toggleDelegateButton = QAction('单行模式', self)
+        self.toggleDelegateButton.triggered.connect(self.toggleDelegate)
 
         saveFile = QAction('保存文件', self)
         saveFile.setShortcut('Ctrl+S')
@@ -163,7 +189,9 @@ class JLineViewer(QMainWindow):
 
         toolbar = self.addToolBar('文件')
         toolbar.addAction(openFile)
-        toolbar.addWidget(self.loadAllButton)  # 将按钮添加到布局中
+        toolbar.addAction(self.reloadButton)
+        toolbar.addAction(self.loadAllButton)  # 将按钮添加到布局中
+        toolbar.addAction(self.toggleDelegateButton)
         # toolbar.addAction(saveFile)
 
         self.statusBar = QStatusBar()
@@ -174,14 +202,24 @@ class JLineViewer(QMainWindow):
         self.treeView.setAlternatingRowColors(True)
         self.treeView.setIndentation(20)
         # self.treeView.setSortingEnabled(True)
-        # self.treeView.setStyleSheet(LargeStrings.treeViewStyles)
-        self.plainTextEdit.setWordWrapMode(QTextOption.WordWrap)
+        self.treeView.setStyleSheet(LargeStrings.treeViewStyles)
+        # self.plainTextEdit.setWordWrapMode(QTextOption.WordWrap)
         self.plainTextEdit.setReadOnly(True)
         self.showMaximized()
 
         self.treeView.setSortingEnabled(False)  # 禁止排序
         self.treeView.setAnimated(False)
-        self.plainTextEdit.textChanged.connect(self.updateJson)  # 连接 textChanged 信号到新的槽函数
+        # self.plainTextEdit.textChanged.connect(self.updateJson)  # 连接 textChanged 信号到新的槽函数
+
+    def toggleDelegate(self):
+        if self.delegate_mode == 'compact':
+            self.treeView.setItemDelegate(ExpandedTextEditDelegate(self.treeView))
+            self.toggleDelegateButton.setText('单行模式')
+            self.delegate_mode = 'expanded'
+        else:
+            self.treeView.setItemDelegate(CompactTextEditDelegate(self.treeView))
+            self.toggleDelegateButton.setText('多行模式')
+            self.delegate_mode = 'compact'
 
     def addPane(self, widget, title):
         layout = QVBoxLayout()
@@ -242,7 +280,8 @@ class JLineViewer(QMainWindow):
             # 2 加载条目数据
             start_time = time.time()
             self.setWindowTitle(f'JLineViewer - {fname}')
-            self.listWidget.addItems([f'{i + 1}. {line.strip()}' for i, line in enumerate(self.lines[:1000])])
+            self.listWidget.addItems([f'{i + 1}. {line.strip()[:1000]}'  # 这里要限制长度，不然遇到长文本软件会巨卡
+                                      for i, line in enumerate(self.lines[:1000])])
             QApplication.processEvents()
 
             # TODO 只有总条目数大于1000时才显示"仅预加载1000条"
@@ -252,6 +291,9 @@ class JLineViewer(QMainWindow):
             else:
                 self.statusBar.showMessage(f"总条目数: {len(self.lines)}, 加载条目耗时: {time.time() - start_time:.2f} 秒")
 
+    def reload(self):
+        self.showDialog(fname=self.lastOpenDir or None)
+
     def loadJson(self, item):
         index = self.listWidget.row(item)
         jsonData = json.loads(self.lines[index])
@@ -260,16 +302,29 @@ class JLineViewer(QMainWindow):
         self.treeView.expandAll()
 
         # 使用自定义的delegate
-        delegate = TextEditDelegate(self.treeView)
-        self.treeView.setItemDelegate(delegate)
+        if self.delegate_mode == 'compact':
+            self.treeView.setItemDelegate(CompactTextEditDelegate(self.treeView))
+        else:
+            self.treeView.setItemDelegate(ExpandedTextEditDelegate(self.treeView))
 
         self.treeView.header().setSectionResizeMode(0, QHeaderView.ResizeToContents)
         self.treeView.header().setSectionResizeMode(1, QHeaderView.Stretch)
 
+        if hasattr(self, 'lastClickedPath'):
+            path = self.lastClickedPath
+            index = self.model.index(path[0][0], path[0][1])  # 从根开始
+            for row, col in path[1:]:
+                index = self.model.index(row, col, index)
+            if index.isValid():
+                self.treeView.setCurrentIndex(index)
+                self.editItem(index)
+
     def loadAllItems(self):
         if not self.allItemsLoaded:
             start_time = time.time()
-            self.listWidget.addItems([f'{i + 1}. {line.strip()}' for i, line in enumerate(self.lines[1000:])])
+            self.listWidget.clear()
+            self.listWidget.addItems([f'{i + 1}. {line.strip()[:1000]}'
+                                      for i, line in enumerate(self.lines)])
             QApplication.processEvents()
             self.statusBar.showMessage(f"全部加载完毕, 总条目数: {len(self.lines)}, 加载耗时: {time.time() - start_time:.2f} 秒")
 
@@ -278,6 +333,11 @@ class JLineViewer(QMainWindow):
         else:
             self.statusBar.showMessage(f"所有条目已经加载完毕。")
 
+    def get_item_full_text(self, item):
+        t = item.text()
+        idx = re.search(r'\d+', t).group()
+        return f'{idx}. {self.lines[int(idx) - 1]}'
+
     def searchItems(self):
         if hasattr(self, 'lines'):
             start_time = time.time()
@@ -285,7 +345,7 @@ class JLineViewer(QMainWindow):
             foundCount = 0
             for i in range(self.listWidget.count()):
                 item = self.listWidget.item(i)
-                if searchText in item.text():
+                if searchText in self.get_item_full_text(item):
                     item.setHidden(False)
                     foundCount += 1
                 else:
@@ -302,7 +362,7 @@ class JLineViewer(QMainWindow):
             foundCount = 0
             for i in range(self.listWidget.count()):
                 item = self.listWidget.item(i)
-                if regexPattern.search(item.text()):  # 使用正则表达式搜索
+                if regexPattern.search(self.get_item_full_text(item)):  # 使用正则表达式搜索
                     item.setHidden(False)
                     foundCount += 1
                 else:
@@ -310,22 +370,42 @@ class JLineViewer(QMainWindow):
             self.statusBar.showMessage(
                 f"总条目数: {len(self.lines)}, 找到: {foundCount}, 搜索耗时: {time.time() - start_time:.2f} 秒")
 
-    def editItem(self, index):
+    def itemToData(self, key_item):
+        value_item = key_item.model().item(key_item.row(), 1)
+        return value_item.text()
+
+    def editItem(self, index=None):
         self.currentlyEditingItem = self.model.itemFromIndex(index)  # 保存当前正在编辑的项
-        target_data = self.currentlyEditingItem.data(Qt.UserRole + 1)  # 获取存储的 JSON 数据
-        if target_data is not None:
-            target_text = json.dumps(target_data, indent=2)  # 把 JSON 对象格式化为字符串
-        else:
-            target_text = self.currentlyEditingItem.text()
+        target_text = self.currentlyEditingItem.text()
         self.plainTextEdit.setPlainText(target_text)
 
+        # 保存路径
+        self.lastClickedPath = []
+        while index.isValid():
+            self.lastClickedPath.append((index.row(), index.column()))
+            index = index.parent()
+        self.lastClickedPath.reverse()
+
     def dictToModel(self, data, parent=None):
         if parent is None:
             parent = QStandardItemModel()
             parent.setHorizontalHeaderLabels(['Key', 'Value'])
+
+        # if isinstance(data, dict):
+        #     for key, value in data.items():
+        #         self.dataToModel(key, value, parent)
+
+        # 判断数据类型，并相应处理
         if isinstance(data, dict):
             for key, value in data.items():
                 self.dataToModel(key, value, parent)
+        elif isinstance(data, list):
+            # 处理列表：创建一个无key的父项，将列表元素作为子项添加
+            self.dataToModel("List", data, parent)
+        else:
+            # 处理基本数据类型：创建一个单独的条目
+            self.dataToModel("Value", data, parent)
+
         return parent
 
     def dataToModel(self, key, value, parent):
@@ -409,7 +489,7 @@ def start_jlineviewer(fname=None):
     if isinstance(fname, list):  # 可以输入一个list字典数据，会转存到临时目录里查看
         tempfile = XlPath.tempfile(suffix='.jsonl')
         tempfile.write_jsonl(fname)
-        fname = tempfile
+        fname = tempfile.as_posix()
     if fname:
         ex.showDialog(fname=fname)
     sys.exit(app.exec_())
@@ -418,4 +498,8 @@ def start_jlineviewer(fname=None):
 if __name__ == '__main__':
     app = QApplication(sys.argv)
     ex = JLineViewer()
+
+    if len(sys.argv) > 1:
+        ex.showDialog(fname=sys.argv[1])
+
     sys.exit(app.exec_())
