@@ -15,17 +15,20 @@ from pyxllib.prog.pupil import check_install_package
 check_install_package('fire')  # 自动安装依赖包
 
 from collections import Counter, defaultdict
-from datetime import date
+from datetime import date, timedelta
 import datetime
 import math
 import os
 import re
 import time
 from io import StringIO
+import csv
 
 import fire
 import pandas as pd
 from tqdm import tqdm
+import requests
+# import requests_cache
 
 from pyxllib.text.pupil import chinese2digits, grp_chinese_char
 from pyxllib.file.xlsxlib import openpyxl
@@ -37,14 +40,109 @@ from pyxllib.data.sqlite import Connection
 from pyxllib.ext.seleniumlib import XlChrome
 
 
+class Xiaoetong:
+    """ 写一个觉观的api
+    """
+
+    def __init__(self):
+        self.token = ''
+
+    def login(self, app_id, client_id, secret_key):
+        """ 登录，获取token
+        """
+        # 启用缓存
+        # requests_cache.install_cache('access_token_cache', expire_after=None)  # 设置缓存过期时间xx（单位：秒）
+        # 接口地址
+        url = "https://api.xiaoe-tech.com/token"
+        params = {
+            "app_id": app_id,
+            "client_id": client_id,
+            "secret_key": secret_key,
+            "grant_type": "client_credential"
+        }
+        response = requests.get(url, params=params)
+        if response.status_code == 200:
+            result = response.json()
+            if result['code'] == 0:
+                # access_token 是小鹅通开放api的全局唯一接口调用凭据，店铺调用各接口时都需使用 access_token ,开发者需要进行妥善保管；
+                self.token = result['data']['access_token']
+            else:
+                raise Exception("Error getting access token: {}".format(result['msg']))
+        else:
+            raise Exception("HTTP request failed with status code {}".format(response.status_code))
+        # return result['data']['access_token']
+    
+    def get_alive_user_list(self, resource_id, page_size=100):
+        """ 获取直播间用户
+        """
+        # 1 获取总页数
+        url = "https://api.xiaoe-tech.com/xe.alive.user.list/1.0.0"  # 接口地址【路径：API列表 -> 直播管理 -> 获取直播间用户列表】
+        data_1 = {
+            "access_token": self.token,
+            "resource_id": resource_id,
+            "page": 1,
+            "page_size": page_size
+        }
+        response_1 = requests.post(url, data=data_1)
+        result_1 = response_1.json()
+        page = math.ceil(result_1['data']['total'] / page_size)  # 页数
+
+        # 2 获取直播间用户数据
+        lst = result_1['data']['list']
+        for i in range(1, page):  # 为什么从1开始，因为第一页的数据上面已经获取到了，这里没必要从新获取一次
+            data = {
+                "access_token": self.token,
+                "resource_id": resource_id,
+                "page": i + 1,
+                "page_size": page_size
+            }
+            response = requests.post(url, data=data)
+            result = response.json()
+            data_1 = result['data']['list']
+            lst += data_1
+            # lst.extend(data_1)
+        return lst
+
+    def get_elock_actor(self, activity_id, page_size=100):
+        """ 获取打卡参与用户
+        """
+        # 获取总页数
+        url = "https://api.xiaoe-tech.com/xe.elock.actor/1.0.0"  # 接口地址【路径：API列表 -> 打卡管理 -> 获取打卡参与用户】
+        data_1 = {
+            "access_token": self.token,
+            "activity_id": activity_id,
+            "page_index": 1,
+            "page_size": page_size
+        }
+        response_1 = requests.post(url, data=data_1)
+        result_1 = response_1.json()
+        page = math.ceil(result_1['data']['count'] / page_size)  # 页数
+        # 获取打卡用户数据
+        lst = result_1['data']['list']
+        for i in range(1, page):  # 为什么从1开始，因为第一页的数据上面已经获取到了，这里没必要从新获取一次
+            data = {
+                "access_token": self.token,
+                "activity_id": activity_id,
+                "page_index": i + 1,
+                "page_size": page_size
+            }
+            response = requests.post(url, data=data)
+            result = response.json()
+            data_1 = result['data']['list']
+            lst += data_1
+            # lst.extend(data_1)
+        return lst
+
+
 class 网课考勤:
-    def __init__(self, today=None):
+    def __init__(self, today=None, *, 表格路径=None):
         self.返款标题 = ''
-        self.表格路径 = r'考勤.xlsx'
+        self.表格路径 = 表格路径 or r'考勤.xlsx'
         self.在线表格 = 'https://docs.qq.com/sheet/DUlF1UnRackJ2Vm5U'  # 生成日报用
         self.开课日期 = '2022-01-08'
         self.视频返款 = [20, 15, 10, 5, 0, 0]  # 直播(当堂)/第1天（当天）/第2天/第3天/第4天/第5天，完成观看的依次返款额。
         self.打卡返款 = {5: 100, 10: 150, 15: 200}  # 打卡满5/10/15次的返款额
+        self.课程链接 = []
         self._init(today)
 
         self.driver = None  # 浏览器脚本
@@ -74,7 +172,10 @@ class 网课考勤:
         self.当天课次2 = min(self.当天课次, self.课次数)
         self.结束课次 = self.当天课次 - len(self.视频返款) + 1  # 这是用于逻辑运算的，可能有负值
         self.结束课次2 = max(0, self.结束课次)
-        self.用户列表 = pd.read_csv(self.get_file('数据表/用户列表导出*.csv'))
+        try:
+            self.用户列表 = pd.read_csv(self.get_file('数据表/用户列表导出*.csv'), dtype={16: str})
+        except:
+            self.用户列表 = None
         # 用户列表 = 用户列表[用户列表['账号状态'] == '正常']
 
         self.考勤表出现次数 = Counter()
@@ -121,23 +222,26 @@ class 网课考勤:
 
         昵称 = tolist(昵称)
         手机号 = tolist(手机号)
-        手机号 = [f'{x}' for x in 手机号]
+        手机号 = [f'{x}' for x in 手机号 if (x and x != 'None')]
 
         # 2 查找所有可能匹配的项目
         ls = []
         for idx, x in self.用户列表.iterrows():
-            logo = False
+            logo = 0
             if '真实姓名' in x:
                 x['姓名'] = x['真实姓名']
 
             if 昵称:
                 if x['昵称'] in 昵称 or x['姓名'] in 昵称:
-                    logo = True
+                    logo += 1
             if 手机号:
                 if check_telphone(x.get('账户绑定手机号'), 手机号) or check_telphone(x.get('最近采集的手机号'), 手机号):
-                    logo = True
+                    logo += 2
             if logo:
-                ls.append(x)
+                ls.append([x, logo])
+
+        ls.sort(key=lambda x: x[1], reverse=True)
+        ls = [x[0] for x in ls]
 
         if debug:
             print('\n'.join(map(self.用户信息摘要, ls)))
@@ -152,7 +256,7 @@ class 网课考勤:
             ls.append('考勤' + str(self.考勤表出现次数[x['用户ID']]) + '次')
         return ', '.join(ls)
 
-    def 匹配用户ID(self):
+    def 匹配用户ID(self, sheet_name='报名表'):
 
         def try2int(x):
             try:
@@ -170,9 +274,13 @@ class 网课考勤:
                 msg[x] = ws.cell2(row, x).value
             return msg
 
-        ws = self.wb['报名表']
+        ws = self.wb[sheet_name]
         for row in tqdm(list(ws.iterrows('真实姓名')), desc='匹配进度'):
+            # todo 这里要考虑相似度整体权重的，而不是几个属性的优先级
             x = todict(row)
+            x['手机号'] = str(x['手机号']).lstrip('`')
+            x['错误手机号'] = str(x['错误手机号']).lstrip('`')
+
             待查手机号 = [x['手机号']]
             t = try2int(x['错误手机号'])
             if t:
@@ -195,6 +303,12 @@ class 网课考勤:
                 if sum(flags) == 1:
                     idx = flags.index(1)
                     用户ID = ls[idx]['用户ID']
+            # 如果用户ID还是空的，则用手机号能匹配上的那条
+            if 用户ID == '':
+                for i, text in enumerate(摘要ls):
+                    if str(x['手机号']) in text:
+                        用户ID = ls[i]['用户ID']
+                        break
             ws.cell2(row, '用户ID', 用户ID)
             ws.cell2(row, '参考信息', '\n'.join(摘要ls))
             row += 1
@@ -207,7 +321,13 @@ class 网课考勤:
             if m := re.match(r'(\d{4}\-\d{2}\-\d{2}).+?课.*?(\d+).+?直播观看详情', f.stem):
                 stat_day, 课次 = date.fromisoformat(m.group(1)), int(m.group(2))
                 skiprows = 1
-            elif m := re.search(r'\d+届.+?(\d+).+?直播用户列表.+?(\d{4}\-\d{2}\-\d{2})', f.stem):  # 新版格式
+            elif m := re.search(r'届(?:念住|觉观).+?(\d+).+?直播用户列表.+?(\d{4}\-\d{2}\-\d{2})', f.stem):
+                stat_day, 课次 = date.fromisoformat(m.group(2)), int(m.group(1))
+                skiprows = 0
+            elif m := re.search(r'第(\d+)堂.+?届(?:念住|觉观).+?直播用户列表.+?(\d{4}\-\d{2}\-\d{2})', f.stem):
+                stat_day, 课次 = date.fromisoformat(m.group(2)), int(m.group(1))
+                skiprows = 0
+            elif m := re.search(r'本体音艺网课-(\d+).+?直播用户列表.+?(\d{4}\-\d{2}\-\d{2})', f.stem):
                 stat_day, 课次 = date.fromisoformat(m.group(2)), int(m.group(1))
                 skiprows = 0
             else:
@@ -275,7 +395,10 @@ class 网课考勤:
             elif '回放' in value:
                 color = RgbFormatter.from_name('黄色')
                 v1 = self.视频返款[0]
-                v2 = self.视频返款[int(re.search(r'第(\d+)天', value).group(1))]
+                idx = min(int(re.search(r'第(\d+)天', value).group(1)),
+                          len(self.视频返款) - 1)
+                v2 = self.视频返款[idx]
+
                 if v2:
                     color = color.light((v1 - v2) / v2)  # 根据返款额度自动变浅
                 else:  # 如果无返款额度
@@ -557,6 +680,12 @@ class 网课考勤:
             except UnicodeDecodeError:
                 pass
 
+        if df is None:
+            try:
+                df = pd.read_csv(files[-1], encoding="ANSI")  # 240226周一11:21，
+            except UnicodeDecodeError:
+                pass
+
         if df is None:
             raise ValueError
 
@@ -683,6 +812,86 @@ class 网课考勤:
         # 保存。一般不用重新再加载self.wb、self.ws，因为这里需求本来基本都是要分段重新执行程序的。
         wb.save(self.表格路径)
 
+    def 匹配交易单号(self, value):
+        """ 更加智能的一条龙匹配操作
+
+        :param value: 需要输入待检索的金额值
+        """
+        # 1 读取账单，账单文件可能不唯一，可能有重复可以自动去重
+        files = XlPath('数据表').rglob_files('*基本账户*.csv')
+        df_list = []
+        for f in files:
+            df = pd.read_csv(f)
+            df_list.append(df)
+        df = pd.concat(df_list, ignore_index=True)
+        # 按照"资金流水单号"去重
+        df = df.drop_duplicates(subset=['资金流水单号'], keep='first')
+        df = df[df['收支类型'] == '`收入']
+        df.reset_index(drop=True, inplace=True)
+
+        # 2 因为情况比较特殊，这里不调用通用的对齐功能，而是定制化写过
+        ws = self.wb['报名表']
+
+        data = ws.iterrows('真实姓名', to_dict=['交易单号'])
+        last_row = -1
+        for i, row in data:
+            last_row = i
+            # 在 df['微信支付业务单号'] 找是否有 row['交易单号']
+            items = df[df['微信支付业务单号'] == row['交易单号']]
+            if items.empty:
+                continue
+
+            # 如果匹配，理论上只有一条
+            item = items.iloc[0]
+            ws.cell2(i, '交易订单号').value = item['业务凭证号'][1:]
+            ws.cell2(i, '订单金额').value = item['收支金额(元)']
+
+            # 在df中去掉所有items
+            df = df.drop(items.index)
+
+        # 3 匹配完后，还有目标金额的数据要列出来
+        
+        # items = df[df['收支金额(元)'] == f'`{value:.2f}']
+        # # todo 这个要识别到'交易单号'中间的日期，降序排序
+        # for idx, row in items.iterrows():
+        #     last_row += 1
+        #     ws.cell2(last_row, '交易单号').value = row['微信支付业务单号']
+        #     ws.cell2(last_row, '交易订单号').value = row['业务凭证号'][1:]
+        #     ws.cell2(last_row, '订单金额').value = row['收支金额(元)']
+
+        # todo：新增测试用：
+        # 初始化一个空的 DataFrame，用于存储遍历结果
+        result_df = pd.DataFrame(columns=['微信支付业务单号', '业务凭证号', '收支金额'])
+        # 遍历符合条件的数据并存储到 result_df 中
+        items = df[df['收支金额(元)'] == f'`{value:.2f}']
+        for idx, row in items.iterrows():
+            # 将 row 中的数据添加到 result_df 中
+            result_df = pd.concat([
+                result_df,
+                pd.DataFrame({
+                    '微信支付业务单号': [row['微信支付业务单号']],
+                    '业务凭证号': [row['业务凭证号'][1:]],  # 去掉业务凭证号的第一个字符
+                    '收支金额': [row['收支金额(元)']]
+                })
+            ], ignore_index=True)
+
+        # 提取日期部分，并将其转为可排序的格式
+        result_df['日期'] = result_df['微信支付业务单号'].str[11:19]  # 提取日期部分，假设日期位于固定位置
+        result_df['日期'] = pd.to_datetime(result_df['日期'], format='%Y%m%d')
+
+        # 按日期降序排序
+        result_df = result_df.sort_values(by='日期', ascending=False).drop(columns=['日期'])
+
+        # 将排序后的 DataFrame 写入到工作表中
+        for idx, row in result_df.iterrows():
+            last_row += 1
+            ws.cell2(last_row, '交易单号').value = row['微信支付业务单号']
+            ws.cell2(last_row, '交易订单号').value = row['业务凭证号']
+            ws.cell2(last_row, '订单金额').value = row['收支金额']
+
+        # 4 保存
+        self.wb.save(self.表格路径)
+
     def __2_自动浏览网页(self):
         pass
 
@@ -706,10 +915,6 @@ class 网课考勤:
         # 然后自己手动操作验证码
         # 以及选择"店铺"
 
-    def 登录微信支付(self):
-        driver = self.ensure_driver()
-        driver.get('https://pay.weixin.qq.com/index.php/core/home/login')
-
     def 下载课次考勤数据(self, 起始课=None, 终止课=None, 文件名前缀=''):
         if 起始课 is None:
             起始课 = max(1, self.结束课次)
@@ -735,7 +940,7 @@ class 网课考勤:
         while True:
             driver.get('https://admin.xiaoe-tech.com/t/basic-platform/downloadCenter')
             if '任务撤回' in driver.locate('//*[@id="downloadsCenter"]/div[4]/div/div[4]/div[2]/table/tbody/tr[1]/'
-                                       'td[9]/div/div/span').text:
+                                           'td[9]/div/div/span').text:
                 time.sleep(1)
             else:
                 time.sleep(2)  # 不能下载太快，还是要稍微等一会
@@ -764,21 +969,123 @@ class 网课考勤:
                 else:
                     time.sleep(1)
 
-    def 批量退款(self):
-        self.driver.get('https://pay.weixin.qq.com/index.php/xphp/cbatchrefund/batch_refund#/pages/index/index')
 
-    def 申请单条退款(self, 凭证号, 退款金额=0, 退款原因=''):
-        driver = self.ensure_driver()
-        driver.get('https://pay.weixin.qq.com/index.php/core/refundapply')
-        driver.locate('//*[@id="app"]/div/div[2]/div[2]/div[2]/div/span/input').send_keys(凭证号)
-        driver.click('//*[@id="applyRefundBtn"]')  # 申请退款
-        driver.locate('//*[@id="app"]/div/div[2]/div[2]/div[3]/div[2]/div/div[1]/div/span[1]/input').send_keys(
-            str(退款金额))
-        driver.locate('//*[@id="textInput"]').send_keys(退款原因)
-        # driver.click('//*[@id="commitRefundApplyBtn"]')  # 建议手动点"提交申请"
+class 网课考勤2(网课考勤):
+    def login_xe(self):
+        self.xe = Xiaoetong()  # 实例化
+        self.xe.login(self.app_id,
+                      self.client_id,
+                      self.secret_key)  # 获取了token
+        
+        return self.xe.token
+    # 依据课程链接，获取资源id（与课次）
+    def 获取课次与资源id(self):
+        课程链接 = self.课程链接[1:]
+        ls_resource_id = [""]
+        for item in 课程链接:  # 课次
+            resource_id = re.search(r"detail\?id=(.+?)\&", item)  # 资源id
+            ls_resource_id.append(resource_id.group(1))
+        return ls_resource_id
+
+    # 获取直播间用户数据
+    def 获取直播间用户数据(self, resource_id, path):
+        if path.is_file():
+            return
+            # 2）获取直播间用户数据：
+        lst = self.xe.get_alive_user_list(resource_id)
+        fieldnames = ['用户ID', '用户昵称', '备注名', '状态', '直播间停留时长(秒)', '直播间停留时长',
+                      '累计观看时长(秒)', '累计观看时长', '直播观看时长(秒)', '直播观看时长', '回放观看时长(秒)',
+                      '回放观看时长', '评论次数', '直播间成交金额']
+        p = path
+        with open(p, mode='w', newline='', encoding='utf-8') as file:
+            writer = csv.DictWriter(file, fieldnames=fieldnames)
+            writer.writeheader()
+            for x in lst:
+                record = {
+                    '用户ID': x['user_id'],
+                    '用户昵称': x['wx_nickname'],
+                    '备注名': None,
+                    '状态': "其它关联权益",
+                    '直播间停留时长(秒)': x['his_online_time'],
+                    '直播间停留时长': str(timedelta(seconds=x['his_online_time'])),
+                    '累计观看时长(秒)': x['his_learn_time'],
+                    '累计观看时长': str(timedelta(seconds=x['his_learn_time'])),
+                    '直播观看时长(秒)': x['his_learning_time'],
+                    '直播观看时长': str(timedelta(seconds=x['his_learning_time'])),
+                    '回放观看时长(秒)': x['his_learned_time'],
+                    '回放观看时长': str(timedelta(seconds=x['his_learned_time'])),
+                    '评论次数': x['comment_num'],
+                    '直播间成交金额': x['user_total_price']
+                }
+                writer.writerow(record)
+
+    def 获取课次列表(self):
+        return list(range(max(self.结束课次2, 1), self.当天课次2 + 1))
+
+    def 下载课程(self):
+        prfx = self.prfx
+        lt = self.获取课次与资源id()
+        for i in tqdm(self.获取课次列表()):
+            resource_id = lt[i]
+            formatted_date = self.today  # datetime.datetime.now().strftime("%Y-%m-%d")
+            path = prfx.format(x=i, y=formatted_date)
+            self.获取直播间用户数据(resource_id, self.root / "数据表" / path)
+
+    # 20240206 新增【针对打卡部分
+    def 获取打卡id(self):
+        """ 依据打卡链接，获取activity_id(打卡id)
+        """
+        打卡链接 = self.打卡链接[1:]
+        ls_activity_id = []
+        for item in 打卡链接:  # 课次
+            activity_id = re.search(r"\?activity_id=(.+?)\&", item)
+            ls_activity_id.append(activity_id.group(1))
+        return ls_activity_id
+
+    def 获取打卡参与用户(self, activity_id, path):
+        # 1）如果路径中已经有了，就跳过
+        if path.is_file():
+            return
+            # 2）获取打卡用户数据：
+        lst = self.xe.get_elock_actor(activity_id)
+        fieldnames = ['用户id', '用户昵称', '打卡昵称', '打卡分组', '姓名', '电话', '最近采集号码', '城市', '微信号',
+                      '打卡天数', '打卡次数', '被点赞数', '被评论数', '被点评数', '被精选数', '参与时间']
+        p = path
+        with open(p, mode='w', newline='', encoding='utf-8') as file:
+            writer = csv.DictWriter(file, fieldnames=fieldnames)
+            writer.writeheader()
+            for x in lst:
+                record = {
+                    '用户id': x['user_id'],
+                    '用户昵称': x['wx_nickname'],
+                    '打卡昵称': x['clock_nickname'],
+                    '打卡分组': None,
+                    '姓名': x['wx_nickname'],
+                    '电话': x['phone'],
+                    '最近采集号码': None,
+                    '城市': x['wx_city'],
+                    '微信号': None,
+                    '打卡天数': x['clock_days'],
+                    '打卡次数': x['clock_days'],
+                    '被点赞数': x['zan_count'],
+                    '被评论数': x['comment_count'],
+                    '被点评数': x['review_count'],
+                    '被精选数': 0,
+                    '参与时间': x['created_at']
+                }
+                writer.writerow(record)
+
+    def 下载打卡数据(self):
+        prfx = "{x}-" + f"《{self.返款标题}技术公益网课【中心教室】-日历打卡学员数据.csv"
+        lt = self.获取打卡id()
+        for i in tqdm(range(1)):
+            activity_id = lt[i]
+            formatted_date = self.today  # datetime.datetime.now().strftime("%Y-%m-%d")
+            path = prfx.format(x=formatted_date)
+            self.获取打卡参与用户(activity_id, self.root / "数据表" / path)
 
 
-class KqDb(Connection):
+class OldKqDb(Connection):
     """ 五一身心行修考勤工具 """
 
     def __init__(self, dbfile='5034.db', wbpath='考勤.xlsx', *args, **kwargs):
@@ -1281,10 +1588,12 @@ class 课次数据:
             return {'value': f'未开始', 'color': '白色'}
 
         # 1 遍历所有文件数据，获得所有最大值
-        data = self.用户观看数据[user_id]
+        data = self.用户观看数据[user_id]  # 这个类的功能框架，已经按照user_id进行了数据分组统计
         for x in data:
+            # 前两条是兼容以前的通用的直播课程数据
             直播分钟 = max(直播分钟, x.get('直播分钟', 0))
             回放分钟 = max(回放分钟, x.get('回放分钟', 0))
+            # 这一条是兼容禅宗特有的百分比进度
             百分比进度 = max(百分比进度, x.get('百分比进度', 0))
 
         # 2 归纳出本课次考勤结论
@@ -1401,11 +1710,6 @@ def 下载课次考勤数据(课程链接, 检查文本=''):
     driver.click('//*[@id="data-export-container"]/div/div[2]/div/div[2]/div[2]/button[2]/span/span')  # 导出
 
 
-def 登录微信支付():
-    driver = get_driver()
-    driver.get('https://pay.weixin.qq.com/index.php/core/home/login')
-
-
 def 聚合读取考勤数据(data_dir, name, judge_minute=30):
     """
     1、在目录"data_folder"下，找前缀包含name的所有文件
