@@ -12,6 +12,12 @@ check_install_package('scp')
 # 对 paramiko 进一步封装的库
 # check_install_package('fabric')
 
+import warnings
+from cryptography.utils import CryptographyDeprecationWarning
+
+# 忽略特定的警告
+warnings.filterwarnings("ignore", category=CryptographyDeprecationWarning)
+
 from collections import defaultdict
 import pathlib
 import re
@@ -104,7 +110,7 @@ class XlSSHClient(paramiko.SSHClient):
         """
         :param str|list server:
         :param map_path: 主要在上传、下载文件的时候，可以用来自动定位路径
-            参考写法：{'C:/': '/'}  # 将D盘映射到服务器位置
+            参考写法：{'C:/': '/'}  # 将C盘映射到服务器位置
         :param int relogin: 当连接失败的时候，可能只是某种特殊原因，可以尝试重新连接
             该参数设置重连的次数
         :param int|float relogin_interval: 每次重连的间隔秒数
@@ -201,7 +207,7 @@ class XlSSHClient(paramiko.SSHClient):
                 try:
                     res = self.client.exec(f'du "{self}" -s -b')
                 except SshCommandError:
-                    res = 0
+                    res = '0'
 
                 sz = int(re.match(r'\d+', res).group())
                 if human_readable:
@@ -259,6 +265,7 @@ class XlSSHClient(paramiko.SSHClient):
 
         # 虽然是基于本地的情况生成随机名称脚本，但一般在服务器也不会冲突，概率特别小
         local_file = XlPath.tempfile()
+        self.exec(f'mkdir -p /tmp/scripts')
         host_file = '/tmp/scripts/' + local_file.name
 
         if file is not None:
@@ -294,7 +301,7 @@ class XlSSHClient(paramiko.SSHClient):
                     return (XlPath(k) / relpath).parent
                 except ValueError:
                     pass
-            raise ValueError('找不到对应的map_path路径映射规则')
+            raise ValueError(f'找不到对应的map_path路径映射规则 {remote_path}')
 
     def __remote_dir(self, local_path, remote_dir):
         if remote_dir is not None:
@@ -414,7 +421,7 @@ class XlSSHClient(paramiko.SSHClient):
             remote_path = self.__remote_dir(local_path, None) / local_path.name
             local_dir = local_path.parent
         else:
-            remote_path = self.Path(remote_path)
+            remote_path = self.Path(remote_path.as_posix())
             local_dir = self.__local_dir(remote_path, local_dir)
             local_path = local_dir / remote_path.name
 
@@ -716,7 +723,7 @@ class XlSSHClient(paramiko.SSHClient):
             self.exec(f'chmod 777 /tmp/download/{package}')  # 下载的包，其他用户也可以读取、执行
         return p
 
-    def reinstall_conda(self, package='Anaconda3-2022.05-Linux-x86_64.sh'):
+    def reinstall_conda(self, package='Anaconda3-2023.03-Linux-x86_64.sh'):
         """ 给当前用户重装anaconda
 
         :param package: 要安装的目标版本，可以修改，以后有时间也可以考虑怎么做成自动找最新版
