@@ -6,7 +6,7 @@
 
 from pyxllib.prog.pupil import check_install_package
 
-check_install_package('win32com', 'pypiwin32')
+check_install_package('win32com', 'pywin32')
 check_install_package('docx', 'python-docx')
 
 import json
