@@ -8,8 +8,11 @@ import os
 import shutil
 from tarfile import TarFile
 # 这个包里的ZipFile是拷贝后修改过的，不影响标准库zipfile里的功能
-import pyxllib.file.packlib.zipfile as zipfile
-from pyxllib.file.packlib.zipfile import ZipFile
+# import pyxllib.file.packlib.zipfile as zipfile
+# from pyxllib.file.packlib.zipfile import ZipFile
+import zipfile
+from zipfile import ZipFile
+import tempfile
 
 from pyxllib.file.specialist import XlPath, reduce_dir_depth
 from pyxllib.prog.pupil import inject_members
@@ -126,6 +129,25 @@ def _unpack_base(packfile, namelist, format=None, extract_dir=None, wrap=0):
         shutil.unpack_archive(packfile, extract_dir, format)
 
 
+def zip_filter_rule(path):
+    p = XlPath(path)
+
+    # 排除特定目录或文件
+    if set(p.parts) & {'__pycache__', '.git', '.idea', 'dist', 'build', '.pytest_cache', '.venv'}:
+        return 0
+
+    # 排除特定文件扩展名
+    if p.suffix in {'.egg-info', '.lock'}:
+        return 0
+
+    # 精细检查内部文件
+    if p.is_dir():
+        return 1
+
+    # 全部包含其他情况
+    return 2
+
+
 class XlZipFile(ZipFile):
 
     def infolist2(self, prefix=None, zipinfo=True):
@@ -151,6 +173,94 @@ class XlZipFile(ZipFile):
     def unpack(self, extract_dir=None, format='zip', wrap=0):
         _unpack_base(self.filename, self.namelist(), format, extract_dir, wrap)
 
+    def write_dir(self, directory, arcname=None, filter_rule=None):
+        """
+        将指定目录（包含子目录）添加到 zip 文件中，并可自定义在 zip 中的存储路径。
+
+        :param directory: 要压缩的目录路径
+        :param arcname: 在 zip 文件中存储的根目录名，
+            None, 默认跟ZipFile.write的arcname一样，直接取输入的目录名
+        :param filter_rule: 过滤规则函数，用于排除不需要的文件。函数应接受文件路径作为参数，
+                           返回0, 1, 或 2，分别表示排除、递归检查、全部包括。
+        """
+        if arcname is None:
+            arcname = directory
+
+        if filter_rule is True:
+            filter_rule = zip_filter_rule
+
+        for root, dirs, files in os.walk(directory):
+            for file in files:
+                file_path = os.path.join(root, file)
+
+                # 调用过滤函数获取过滤状态
+                if filter_rule:
+                    result = filter_rule(file_path)
+                    if result == 0:  # 明确不要
+                        continue
+                    elif result == 1:  # 需要递归精细检查
+                        pass  # 继续执行文件写入逻辑
+                    elif result == 2:  # 全部包含
+                        # 计算文件在 zip 文件中的相对路径
+                        zip_path = os.path.join(arcname, os.path.relpath(file_path, directory))
+                        self.write(file_path, zip_path)
+                        continue
+
+                # 默认情况下，递归处理
+                zip_path = os.path.join(arcname, os.path.relpath(file_path, directory))
+                self.write(file_path, zip_path)
+
+            for dir in dirs[:]:
+                dir_path = os.path.join(root, dir)
+
+                if filter_rule:
+                    result = filter_rule(dir_path)
+                    if result == 0:  # 明确不要
+                        dirs.remove(dir)  # 排除整个目录
+                    elif result == 2:  # 全部包含
+                        # 添加整个目录
+                        for root2, _, files2 in os.walk(dir_path):
+                            for file2 in files2:
+                                file_path = os.path.join(root2, file2)
+                                zip_path = os.path.join(arcname, os.path.relpath(file_path, directory))
+                                self.write(file_path, zip_path)
+                        dirs.remove(dir)  # 递归中不再处理此目录
+
+    def write_path(self, path, arcname=None, filter_rule=None):
+        """ 封装的同时支持文件或目录的操作 """
+        if XlPath(path).is_dir():
+            self.write_dir(path, arcname, filter_rule)
+        elif XlPath(path).is_file():
+            self.write(path, arcname)
+        else:
+            raise ValueError
+
+    def xlwrite(self, path, arcname=None, filter_rule=True):
+        self.write_path(path, arcname, filter_rule)
+
+    @classmethod
+    def create(cls, name=None):
+        """ 在临时目录下新建一个目录，然后放一个压缩包文件 """
+        # 如果没有提供 name，则生成一个临时的文件名
+        if name is None:
+            name = tempfile.mktemp(suffix=".zip").split('/')[-1]  # 提取生成的文件名
+
+        # 创建临时目录（因为可能有并发操作，每个处理都建立一个目录更安全）
+        temp_dir = tempfile.mkdtemp()
+        temp_zip_path = f"{temp_dir}/{name}"
+
+        # 创建 ZipFile 对象
+        zipf = ZipFile(temp_zip_path, 'w')
+        return zipf
+
+    def fastapi_resp(self):
+        """ 返回供fastapi后端使用的文件接口 """
+        from fastapi.responses import FileResponse
+
+        self.close()
+        return FileResponse(self.filename, media_type='application/zip',
+                            filename=XlPath(self.filename).name)
+
 
 class XlTarFile(TarFile):
 
@@ -162,7 +272,7 @@ inject_members(XlZipFile, ZipFile)
 inject_members(XlTarFile, TarFile)
 
 
-def unpack_archive(filename, extract_dir=None, format=None, *, wrap=0):
+def unpack_archive(filename, extract_dir=None, format=None, *, wrap=1):
     """ 对shutil.unpack_archive的扩展，增加了一个wrap的接口功能 """
     if format is None:
         format = shutil._find_unpack_format(str(filename).lower())
@@ -176,12 +286,14 @@ def unpack_archive(filename, extract_dir=None, format=None, *, wrap=0):
         shutil.unpack_archive(filename, extract_dir, format)
 
 
-def compress_to_zip(source_path, target_zip_path=None):
-    """
-    压缩指定的文件或文件夹为ZIP格式。
+def compress_to_zip(source_path, target_zip_path=None, wrap=None,
+                    ignore_func=None):
+    """ 压缩指定的文件或文件夹为ZIP格式。
 
     :param str source_path: 要压缩的文件或文件夹路径
     :param str target_zip_path: 目标ZIP文件路径（可选）
+    :param str wrap: 在ZIP文件内部创建的目录名，所有内容将被放在这个目录下
+    :param func ignore_func: 忽略文件的函数，输入目录或文件的路径，返回True表示不取用
     """
     # 根据输入路径生成默认的目标ZIP文件路径
     if target_zip_path is None:
@@ -195,14 +307,24 @@ def compress_to_zip(source_path, target_zip_path=None):
         # 如果是文件夹，则遍历文件夹中的所有文件和子文件夹
         if os.path.isdir(source_path):
             for root, _, files in os.walk(source_path):
+                if ignore_func and ignore_func(root):
+                    continue
+
                 for file in files:
+                    if ignore_func and ignore_func(file):
+                        continue
+
                     file_path = os.path.join(root, file)
                     # 计算文件在ZIP中的相对路径
                     arcname = os.path.relpath(file_path, source_path)
+                    if wrap:
+                        arcname = os.path.join(wrap, arcname)
                     zipf.write(file_path, arcname)
         # 如果是文件，则直接将文件添加到ZIP中
         elif os.path.isfile(source_path):
             arcname = os.path.basename(source_path)
+            if wrap:
+                arcname = os.path.join(wrap, arcname)
             zipf.write(source_path, arcname)
 
     return target_zip_path
