@@ -28,7 +28,7 @@ class SingletonForEveryClass(type):
 
 
 class SingletonForEveryInitArgs(type):
-    """Python单例模式(Singleton)的N种实现 - 知乎: https://zhuanlan.zhihu.com/p/37534850
+    """ Python单例模式(Singleton)的N种实现 - 知乎: https://zhuanlan.zhihu.com/p/37534850
 
     注意！注意！注意！重要的事说三遍！
     这里的单例类不是传统意义上的单例类。
@@ -251,6 +251,42 @@ def human_readable_size(n):
         return f'{round_int(n)}TB'
 
 
+def human_readable_number(value, base_type='K', precision=4):
+    """ 数字美化输出函数
+
+    :param float|int value: 要转换的数值
+    :param int precision: 有效数字的长度
+    :param str base_type: 进制类型，'K'为1000进制, 'KB'为1024进制（KiB同理）, '万'为中文万进制
+    :return: 美化后的字符串
+    """
+    if value is None:
+        return ''
+
+    if abs(value) < 1:
+        return f'{value:.{precision}g}'
+
+    # 设置不同进制的单位和基数
+    units, base = {
+        'K': (['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'], 1000),
+        'KB': (['', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'], 1024),
+        'KiB': (['', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB'], 1024),
+        '万': (['', '万', '亿', '万亿', '亿亿'], 10000),
+        '秒': (['秒', [60, '分'], [60, '时'], [24, '天'], [7, '周'], [4.345, '月'], [12, '年']], 60),
+    }.get(base_type, ([''], 1))  # 默认为空单位和基数1
+
+    x, i = abs(value), 0
+    while x >= base and i < len(units) - 1:
+        x /= base
+        i += 1
+        if isinstance(units[i], list):
+            base = units[i][0]
+            units[i] = units[i][1]
+
+    x = f'{x:.{precision}g}'  # 四舍五入到指定精度
+    prefix = '-' if value < 0 else ''  # 负数处理
+    return f"{prefix}{x}{units[i]}"
+
+
 class CvtType:
     """ 这些系列的转换函数，转换失败统一报错为ValueError
 
@@ -371,3 +407,45 @@ def funcmsg(func):
         else:
             return f'装饰器：{type(func)}，无法定位'
     return f'函数名：{func.__name__}，来自文件：{func.__code__.co_filename}，所在行号={func.__code__.co_firstlineno}'
+
+
+def set_global_var(name, value):
+    """ 设置某个全局变量，一般用在一些特殊的需要跨作用域进行调试的场景
+    切勿！切勿！切勿用于正式功能，否则会导致难以维护控制的功能代码
+
+    为了避免和某些关键的全局变量名冲突，这里的变量命令统一会加上pyxllib_的前缀
+    """
+    g = globals()
+    name = f'pyxllib_{name}'
+    g[name] = value
+
+
+def get_global_var(name, default_value=None):
+    """ 获取某个全局变量的值 """
+    g = globals()
+    name = f'pyxllib_{name}'
+    if name not in g:
+        g[name] = default_value
+    return g[name]
+
+
+def convert_to_json_compatible(d, custom_converter=None):
+    """ 递归地将字典等类型转换为JSON兼容格式。对于非标准JSON类型，使用自定义转换器或默认转换为字符串。
+
+    :param d: 要转换的字典或列表。
+    :param custom_converter: 自定义转换函数，用于处理非标准JSON类型的值。
+    :return: 转换后的字典或列表。
+
+    todo 这个函数想法是好的，但总感觉精确性中，总容易有些问题的，需要更多的考察测试
+    """
+    if custom_converter is None:
+        custom_converter = str
+
+    if isinstance(d, dict):  # defaultdict呢？
+        return {k: convert_to_json_compatible(v, custom_converter) for k, v in d.items()}
+    elif isinstance(d, list):
+        return [convert_to_json_compatible(v, custom_converter) for v in d]
+    elif isinstance(d, (int, float, str, bool)) or d is None:
+        return d
+    else:
+        return custom_converter(d)
