@@ -6,9 +6,11 @@
 
 
 """ 封装一些代码开发中常用的功能，工程组件 """
-import builtins
+
 from collections import Counter
-import ctypes
+from concurrent.futures import ThreadPoolExecutor
+from urllib.parse import urlparse
+import builtins
 import datetime
 import functools
 import hashlib
@@ -21,20 +23,23 @@ import math
 import os
 import pprint
 import queue
+import random
+import re
 import signal
 import socket
-import socketserver
 import subprocess
 import sys
 import tempfile
 import threading
 import time
 import traceback
-from urllib.parse import urlparse
 
 from pyxllib.prog.newbie import classproperty, typename
 
 
+# from loguru import logger
+
+
 def system_information():
     """主要是测试一些系统变量值，顺便再演示一次Timer用法"""
 
@@ -226,7 +231,7 @@ class DictTool:
 
         :param keys: 可以输入另一个字典，也可以输入一个列表表示要删除的键值清单
 
-        :return: dict_ -= keys
+        :return: dict2 = dict_ - keys
         """
         if isinstance(keys, dict):
             keys = keys.keys()
@@ -241,7 +246,7 @@ class DictTool:
 
         keys可以输入另一个字典，也可以输入一个列表表示要删除的键值清单
 
-        :return: dict_ -= keys
+        效果相当于 dict_ -= keys
         """
         if isinstance(keys, dict):
             keys = keys.keys()
@@ -403,8 +408,9 @@ def check_install_package(package, speccal_install_name=None, *, user=False):
         subprocess.check_call(cmds)
 
 
-def run_once(distinct_mode=0, *, limit=1):
+def run_once(distinct_mode=0, *, limit=1, debug=False):
     """ 装饰器，装饰的函数在一次程序里其实只会运行一次
+
     :param int|str distinct_mode:
         0，默认False，不区分输入的参数值（包括cls、self），强制装饰的函数只运行一次
         'str'，设为True或1时，仅以字符串化的差异判断是否是重复调用，参数不同，会判断为不同的调用，每种调用限制最多执行limit次
@@ -446,6 +452,7 @@ def run_once(distinct_mode=0, *, limit=1):
             if x[0] < limit:
                 res = func(*args, **kwargs)
                 x = counter[tag] = [x[0] + 1, res]
+
             return x[1]
 
         return wrapper
@@ -476,9 +483,23 @@ def set_default_args(*d_args, **d_kwargs):
     return decorator
 
 
-def utc_now(offset_hours=8):
-    """ 有的机器可能本地时间设成了utc0，可以用这个方式，获得准确的utc8时间 """
-    return datetime.datetime.utcnow() + datetime.timedelta(hours=offset_hours)
+def utc_now(offset_hours=8, microseconds=0):
+    """ 有的机器可能本地时间设成了utc0，可以用这个方式，获得准确的utc8时间
+
+    :param microseconds: 微秒，如果不指定（None），就是当前时间的微秒
+    """
+    dt = datetime.datetime.utcnow()
+    dt += datetime.timedelta(hours=offset_hours)
+
+    if microseconds is not None:
+        dt = dt.replace(microsecond=microseconds)
+
+    return dt
+
+
+def utc_now2(offset_hours=8):
+    """ 转字符串格式 """
+    return utc_now().isoformat(' ', timespec='seconds')
 
 
 def utc_timestamp(offset_hours=8):
@@ -491,7 +512,7 @@ class Timeout:
     """ 对函数等待执行的功能，限制运行时间
 
     【实现思路】
-    1、最简单的方式是用signal.SIGALRM实现
+    1、最简单的方式是用signal.SIGALRM实现（包括三方库timeout-decorator也有这个局限）
         https://stackoverflow.com/questions/2281850/timeout-function-if-it-takes-too-long-to-finish
         但是这个不支持windows系统~~
     2、那windows和linux通用的做法，就是把原执行函数变成一个子线程来运行
@@ -502,6 +523,9 @@ class Timeout:
         是用一个Timer计时器子线程计时，当timeout超时，使用信号机制给主线程抛出一个异常
             ① 注意，不能子线程直接抛出异常，这样影响不了主线程
             ② 也不能直接抛出错误signal，这样会强制直接中断程序。应该抛出TimeoutError，让后续程序进行超时逻辑的处理
+            ③ 这里是让子线程抛出信号，主线程收到信号后，再抛出TimeoutError
+
+    注意：这个函数似乎不支持多线程
     """
 
     def __init__(self, seconds):
@@ -533,6 +557,9 @@ class Timeout:
         return wrapper
 
     def __enter__(self):
+        if self.seconds == 0:  # 可以设置0来关闭超时功能
+            return
+
         def overtime(signum, frame):
             raise TimeoutError(f'with 上下文代码块运行超时 > [{self.seconds} 秒]')
 
@@ -541,10 +568,15 @@ class Timeout:
         def send_signal():
             signal.raise_signal(signal.SIGABRT)
 
+        # 挂起一个警告器，如果"没人"管它，self.seconds就会抛出错误
         self.alarm = threading.Timer(self.seconds, send_signal)
         self.alarm.start()
 
     def __exit__(self, exc_type, exc_val, exc_tb):
+        if self.seconds == 0:
+            return
+
+        # with已经运行完了，马上关闭警告器
         self.alarm.cancel()
 
 
@@ -572,6 +604,7 @@ def inject_members(from_obj, to_obj, member_list=None, *,
     # 把XlDocxTable的成员方法绑定到docx.table.Table里
     >> inject_members(XlDocxTable, docx.table.Table)
 
+    240826周一，其他可参考学习的三方现成工具库：from fastcore.foundation import patch
     """
     # 1 整理需要注入的方法清单
     dst = set(dir(to_obj))
@@ -599,20 +632,6 @@ def inject_members(from_obj, to_obj, member_list=None, *,
             logging.warning(f'Conflict of the same name! {to_obj}.{x}')
 
 
-def find_free_ports(count=1):
-    """ 随机获得可用端口
-
-    :param count: 需要的端口数量（会保证给出的端口号不重复）
-    :return: list
-    """
-    ports = set()
-    while len(ports) < count:
-        with socketserver.TCPServer(("localhost", 0), None) as s:
-            ports.add(s.server_address[1])
-
-    return list(ports)
-
-
 def __debug系列():
     pass
 
@@ -804,8 +823,18 @@ class DPrint:
         return sep.join(msg)
 
 
-def format_exception(e):
-    return ''.join(traceback.format_exception(etype=type(e), value=e, tb=e.__traceback__))
+def format_exception(e, mode=3):
+    if mode == 1:
+        # 仅获取异常类型的名称
+        text = ''.join(traceback.format_exception_only(type(e), e)).strip()
+    elif mode == 2:
+        # 获取异常类型的名称和附加的错误信息
+        text = f"{type(e).__name__}: {e}"
+    elif mode == 3:
+        text = ''.join(traceback.format_exception(type(e), e, e.__traceback__))
+    else:
+        raise ValueError
+    return text
 
 
 def prettifystr(s):
@@ -875,7 +904,7 @@ class OutputLogger(logging.Logger):
     也能在需要的时候指定文件路径，会自动将结果存储到文件中。
     """
 
-    def __init__(self, name='OutputLogger', log_file=None, output_to_console=True):
+    def __init__(self, name='OutputLogger', *, log_file=None, log_mode='a', output_to_console=True):
         """
         :param str name: 记录器的名称。默认为 'OutputLogger'。
         :param log_file: 日志文件的路径。默认为 None，表示不输出到文件。
@@ -892,24 +921,31 @@ class OutputLogger(logging.Logger):
                                       '%Y-%m-%d %H:%M:%S')
 
         # 提前重置为空文件
-        with open(log_file, 'w') as f:
-            f.write('')
+        if log_file is not None:
+            if not os.path.isfile(log_file) or log_mode == 'w':
+                with open(log_file, 'w', encoding='utf8') as f:
+                    f.write('')
 
         # 创建文件日志处理器
         if log_file:
             file_handler = logging.FileHandler(log_file, encoding='utf-8')
-            file_handler.setLevel(logging.INFO)
+            file_handler.setLevel(logging.DEBUG)  # 日志文件是最详细级别都记录
             file_handler.setFormatter(formatter)
             self.addHandler(file_handler)
 
         # 创建命令行日志处理器
         if output_to_console:
             console_handler = logging.StreamHandler()
-            console_handler.setLevel(logging.INFO)
+            console_handler.setLevel(logging.WARNING)  # 有些太详细的问题，不想写在控制台，而是写到文件
             console_handler.setFormatter(formatter)
             self.addHandler(console_handler)
 
+        # 只输出到控制台：标准库的print
+        # 只输出到文件：debug, info
+        # 同时输出到控制台和文件：warning, error, critical, print
+
     def print(self, *args, **kwargs):
+        """ 使用print机制，会同时输出到控制台和日志文件 """
         msg = print2string(*args, **kwargs)
 
         if self.output_to_console:
@@ -921,46 +957,15 @@ class OutputLogger(logging.Logger):
 
         return msg
 
+    def tprint(self, *args, **kwargs):
+        """ 带时间戳的print """
+        self.print(utc_now2(), *args, **kwargs)
 
-class MultiProcessLauncher:
-    """ 注意这个类暂时只有linux能使用 """
-
-    def __init__(self):
-        self.workers = []
-
-    def add_process(self, cmd, name=None, **kwargs):
-        if name is None:
-            name = cmd.split()[0]
-
-        p = subprocess.Popen(cmd.split(), preexec_fn=self._set_pdeathsig(signal.SIGTERM))
-        worker = {'name': name, 'process': p, 'command': cmd}
-        worker.update(kwargs)
-        self.workers.append(worker)
-
-    def add_python_module(self, module, args='', name=None):
-        """ 添加并启动一个Python模块作为后台进程。
-
-        :param module: 要执行的Python模块名（python -m 后面的部分）
-        :param args: 模块的参数
-        :param name: 进程的名称，默认为模块名
-        """
-        cmd = f'{sys.executable} -m {module} {args}'
-        self.add_process(cmd, name=name)
-
-    def stop_all(self):
-        """停止所有后台进程"""
-        for worker in self.workers:
-            worker.process.terminate()
-        self.workers = []
-
-    def _set_pdeathsig(self, sig=signal.SIGTERM):
-        """ 在主服务退出时，这些进程也会全部自动关闭 """
-
-        def callable():
-            libc = ctypes.CDLL("libc.so.6")
-            return libc.prctl(1, sig)
-
-        return callable
+    def log_json(self, data):
+        """ 类似print，但是是把数据按照json的格式进行记录整理，更加结构化，方便后期处理 """
+        data['time'] = utc_timestamp()
+        msg = json.dumps(data, ensure_ascii=False, default=str)
+        self.print(msg)
 
 
 def xlmd5(content):
@@ -977,9 +982,194 @@ def xlmd5(content):
 
 @run_once()
 def get_hostname():
-    return socket.getfqdn()
+    hostname = socket.getfqdn()
+    return hostname
+
+
+@run_once()
+def get_hostname2():
+    """ 更加定制化的操作 """
+    hostname = socket.getfqdn()
+    hostname = hostname.replace('-', '_')
+    hostname = hostname.split('.')[0]
+    return hostname
 
 
 @run_once()
 def get_username():
     return os.path.split(os.path.expanduser('~'))[-1]
+
+
+class XlThreadPoolExecutor(ThreadPoolExecutor):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.futures = []
+
+    def submit(self, *args, **kwargs):
+        future = super().submit(*args, **kwargs)
+        self.futures.append(future)
+        return future
+
+    def yield_result(self, timeout=None):
+        for future in self.futures:
+            yield future.result(timeout=timeout)
+
+
+def shuffle_dict_keys(d):
+    keys = list(d.keys())
+    random.shuffle(keys)
+    d = {k: d[k] for k in keys}
+    return d
+
+
+def generate_int_hash_from_str(s):
+    """ 对字符串使用md5编码，然后转出一个数值哈希，一般是用来进行随机分组
+    比如获得一个整数后，对3取余，就是按照余数为0、1、2的情况分3组
+    """
+    return int(hashlib.md5(s.encode()).hexdigest(), 16)
+
+
+def get_groupid_from_string(s, n_groups):
+    """ 通过计算一个字符串的哈希值来对其进行分组，需要提前知道总组别数n_groups """
+    hash_value = generate_int_hash_from_str(s)
+    return hash_value % n_groups
+
+
+def safe_div(a, b):
+    """ 安全除法，避免除数为0的情况
+
+    :param a: 被除数
+    :param b: 除数
+    :return: a/b，如果b为0，返回0
+    """
+    if b == 0:
+        return a / sys.float_info.epsilon
+    else:
+        return a / b
+
+
+def inplace_decorate(parent, func_name, wrapper):
+    """ 将指定的函数替换为装饰器版本
+    允许在运行时动态地将一个函数或方法替换为其装饰版本。通常用于添加日志、性能测试、事务处理等。
+    （既然可以写成装饰版本，相当于其实要完全替换成另外的函数也是可以的）
+
+    当然，因为py一切皆对象，这里处理的不是函数，而是其他变量等对象也是可以的
+
+    这个功能跟直接把原代码替换修改了还是有区别的，如果原函数在被这个装饰之前，已经被其他地方调用，或者被装饰器补充，
+        太晚使用这个装饰，并不会改变前面已经运行、被捕捉的情况
+        遇到这种情况，也可以考虑在原函数定义后，直接紧接着加上这个函数重置
+
+    对于类成员方法，直接用这个设置可能也不行，只能去改源码了
+        比如要给函数加计算时间的部分，可以考虑使用 get_global_var 等来夸作用域记录时间数据
+    """
+
+    if hasattr(parent, func_name):
+        if callable(wrapper):  # 对函数的封装
+            original_func = getattr(parent, func_name)
+
+            @functools.wraps(original_func)
+            def decorated_func(*args, **kwargs):
+                return wrapper(original_func, *args, **kwargs)
+
+            setattr(parent, func_name, decorated_func)
+        else:  # 对数值的封装，不过如果是数值，其实也没必要调用这个函数，直接赋值就好了
+            return setattr(parent, func_name, wrapper)
+    else:  # 否则按照字典的模式来处理
+
+        original_func = parent[func_name]
+
+        @functools.wraps(original_func)
+        def decorated_func(*args, **kwargs):
+            return wrapper(original_func, *args, **kwargs)
+
+        parent[func_name] = decorated_func
+
+
+def check_counter(data, top_n=10):
+    """ 将一个数据data转为Counter进行频数分析 """
+    # 1 如果是list、tuple类型，需要转counter
+    if isinstance(data, (list, tuple)):
+        data = Counter(data)
+    if not isinstance(data, Counter):
+        raise ValueError(f'输入的数据类型不对，应该是Counter类型，而不是{typename(data)}')
+
+    # 2 列出出现次数最多的top_n条目
+    # 打印基本统计信息
+    total_items = sum(data.values())
+    print(f"总条目数: {total_items}")
+
+    if top_n > 0:
+        top_items = data.most_common(top_n)
+        max_n = len(data)
+        print(f"出现次数最多的{min(top_n, max_n)}/{max_n}条数据（频率）:")
+        for item, count in top_items:
+            print(f"\t{item}\t{count}")
+
+    # 3 打印基本统计信息
+    # 对原始Counter的计数值进行再计数
+    count_frequencies = Counter(data.values())
+
+    # 打印各计数值出现的次数
+    print("各计数值出现的次数，频率的频率（频率分布）:")
+    for count, frequency in count_frequencies.most_common():
+        print(f"\t{count}\t{frequency}")
+
+
+def tprint(*args, **kwargs):
+    """ 带时间戳的print """
+    print(utc_now2(), *args, **kwargs)
+
+
+def is_valid_identifier(name):
+    """ 判断是否是合法的标识符 """
+    return re.match(r'^[a-zA-Z_][a-zA-Z0-9_]*$', name)
+
+
+def get_number_width(n):
+    """ 判断数值n的长度
+
+    参考资料：https://jstrieb.github.io/posts/digit-length/
+
+    >>> get_number_width(0)
+    1
+    >>> get_number_width(9)
+    1
+    >>> get_number_width(10)
+    2
+    >>> get_number_width(97)
+    2
+    """
+    # assert n > 0
+    # return math.ceil(math.log10(n + 1))
+
+    return 1 if n == 0 else (math.floor(math.log10(n)) + 1)
+
+
+def aligned_range(start, stop=None, step=1):
+    """ 返回按照域宽对齐的数字序列 """
+    if stop is None:
+        start, stop = 0, start
+
+    max_width = get_number_width(stop - step)
+    format_str = '{:0' + str(max_width) + 'd}'
+
+    return [format_str.format(i) for i in range(start, stop, step)]
+
+
+def percentage_and_value(numbers, precision=2, *, total=None, sep='.'):
+    """ 对输入的一串数值，转换成一种特殊的表达格式 "百分比.次数"
+
+    :param list numbers: 数值列表
+    :param int precision: 百分比的精度（小数点后的位数），默认为 2
+    :param int total: 总数，如果不输入，则默认为输入数值的和
+    :param str sep: 分隔符
+    :return: 整数部分是比例，小数部分是原始数值的整数部分
+    """
+    if total is None:
+        total = sum(numbers)
+    width = get_number_width(total)
+    result = []
+    for num in numbers:
+        percent = safe_div(num, total) * 10 ** precision
+        result.append(f"{percent:.0f}{sep}{num:0{width}d}")
+    return result
