@@ -32,6 +32,7 @@ from pyxllib.algo.pupil import SearchBase
 from pyxllib.text.newbie import xldictstr
 from pyxllib.text.pupil import shorten, ensure_gbk, BookContents, strwidth, grp_chinese_char
 from pyxllib.file.specialist import File, Dir, get_etag
+from pyxllib.text.jinjalib import get_jinja_template
 
 
 class XlBs4Tag(bs4.element.Tag):
@@ -144,7 +145,9 @@ class XlBs4Tag(bs4.element.Tag):
             # 下个结点
             t = t.next_element
             i += 1
-        df1 = pd.DataFrame.from_records(ls1, columns=['element序号', '层级', '结构', '父结点', '当前结点', '属性值/字符串值', '直接子结点结构'])
+        df1 = pd.DataFrame.from_records(ls1,
+                                        columns=['element序号', '层级', '结构', '父结点', '当前结点', '属性值/字符串值',
+                                                 '直接子结点结构'])
         df2 = pd.DataFrame.from_records(ls2, columns=['序号', 'element序号', '当前结点', '属性名', '属性值'])
         return df1, df2
 
@@ -255,9 +258,9 @@ class XlBs4Tag(bs4.element.Tag):
         for h in self.find_all(re.compile(r'h\d')):
             if size:
                 part_size = h.section_text_size(size, fmt=True)
-                bc.add(int(h.name[1]), h.get_text().replace('\n', ' '), part_size)
+                bc.add(int(h.name[1]), h.get_text().replace('\n', ' ').strip(), part_size)
             else:
-                bc.add(int(h.name[1]), h.get_text().replace('\n', ' '))
+                bc.add(int(h.name[1]), h.get_text().replace('\n', ' ').strip())
 
         if 'page' not in kwargs:
             kwargs['page'] = size
@@ -668,14 +671,6 @@ class HtmlParser:
             getattr(self, method)(x)
 
 
-def get_jinja_template(name, **kwargs):
-    from jinja2 import Environment
-    from pyxllib.file.specialist import XlPath
-
-    template = Environment(**kwargs).from_string((XlPath(__file__).parent / f'templates/{name}').read_text())
-    return template
-
-
 def concat_htmlbody(ls):
     """ 对多份网页内容中的body进行拼接
     """
