@@ -204,7 +204,7 @@ class XlAiClient:
         self._priu_header = None
         self._priu_host = None
 
-        # 2 如果环境变量预存了账号信息，自动加载使用
+        # 2 (旧)如果环境变量预存了账号信息，自动加载使用
         accounts = XlOsEnv.get('XlAiAccounts', decoding=True)
         if accounts and auto_login:
             if 'aipocr' in accounts:
@@ -214,6 +214,17 @@ class XlAiClient:
             if 'priu' in accounts:
                 self.login_priu(**accounts['priu'], check=check)
 
+        # 3 (新)增设了更规范的环境变量名
+        if auto_login and os.environ.get("XL_API_AIP_ID"):
+            self.login_aipocr(os.environ.get("XL_API_AIP_ID"),
+                              os.environ.get("XL_API_AIP_KEY"),
+                              os.environ.get("XL_API_AIP_SKEY"))
+        if auto_login and os.environ.get("XL_API_MATHPIX_ID"):
+            self.login_mathpix(os.environ.get("XL_API_MATHPIX_ID"),
+                               os.environ.get("XL_API_MATHPIX_KEY"))
+        if auto_login and os.environ.get("XL_API_PRIU_TOKEN"):
+            self.login_priu(os.environ.get("XL_API_PRIU_TOKEN"))
+
     def __A1_登录账号(self):
         pass
 
@@ -268,7 +279,7 @@ class XlAiClient:
             connent = False
             for host in hosts:
                 try:
-                    if '欢迎来到 厦门理工' in requests.get(f'{host}/test_page', timeout=5).text:
+                    if requests.get(f'{host}/healthz', timeout=5).status_code == 200:
                         self._priu_host = host
                         connent = True
                         break
@@ -442,7 +453,8 @@ class XlAiClient:
         """ 通用文字识别（高精度版）: https://cloud.baidu.com/doc/OCR/s/1k3h7y3db """
         sz = 10 * 1024 ** 2
         buffer, ratio = self.adjust_image(image, max_length=8192, limit_b64buffer_size=sz, to_buffer=True)
-        result_dict = self.run_aipocr_with_db(self._aipocr.basicAccurate, buffer, options, save_buffer_threshold_size=sz)
+        result_dict = self.run_aipocr_with_db(self._aipocr.basicAccurate, buffer, options,
+                                              save_buffer_threshold_size=sz)
         result_dict = ToLabelmeLike.list_word(result_dict, 1 / ratio, 'words_result', 'words_result_num')
         return result_dict
 
@@ -1113,6 +1125,8 @@ class XlAiClient:
         except requests.exceptions.Timeout:
             # 为了方便处理，统一也返回ConnectionError
             raise requests.exceptions.ConnectionError()
+        except Exception:
+            raise requests.exceptions.ConnectionError()
 
         if r.status_code == 200:
             res = json.loads(r.text)
